==> RealiBoxTestBiz-0.6.6.tar.gz: RealiBoxTestBiz-0.6.6/realiboxtestbiz/utils/test_collection.py <==
'''
收集用例case_collection：
目前用例收集逻辑：
    通过pytest钩子采集每轮执行的用例，通过与 CaseCollection 表中进行diff。
    表中没有则添加。遗留问题：
        需要全量同步方法，否则未调用过的用例不会采集，已删除的用例无记录。目前可以通过刷库解决，
        但是需要在实现前端调用用例之前修复该问题

任务结果上报task_report：

'''
from ..utils.realiboxtestTables import CaseCollection
from sqlalchemy import and_
from ..utils.realiboxtestTables import TaskReport
from sqlalchemy.orm import sessionmaker
import uuid
import importlib
import base64
from _pytest.main import Session
from _pytest.python import Function, Module, Package
from .. import cerebro


class Collection:

    def __init__(self, engine):
        # 创建一个mysql链接session
        Session = sessionmaker(engine)
        # 打开会话对象 Session
        self.db_session = Session()
        self.case_info_list = []

    def _case_unique_code(self, obj, tmp):
        '''生成用例唯一码
        params fn：用例函数对象
        params tmp：一个类， 其属性用于记录递归过程用例信息
        '''
        # Session为最后一个节点不处理，跳出递归
        if isinstance(obj, Session):
            return
        # 节点类型
        node_type = None
        # 当obj为Function
        if isinstance(obj, Function):
            node_type = 'function'

        #   当obj是moudule时，为用例文件节点，获取__doc__为用例名称
        if isinstance(obj, Module) and (not tmp.case_name):
            node_type = 'module'
            tmp.case_name = obj.obj.__doc__

        #   当obj是Package时，为路径节点，使用tmp记录其层级关系， 第0级为模块， 获取__doc__为模块名称
        if isinstance(obj, Package):
            node_type = 'package'
            # 判断当前目录是否与一级目录相同
            second_node = '.'.join(obj.nodeid.split('/')[:-2])
            if not second_node == tmp.first_node:
                tmp.second_node = second_node

        # 每进入一个层级则创建一个类，将上一个层级的类加载到这个类的node属性
        current_node = type('Node', (), {
            'next': tmp.case_structure,
            'type': node_type
        })
        # 将当前节点加载到Tmp中
        tmp.case_structure = current_node
        self._case_unique_code(obj.parent, tmp)

    def case_info_list_collect(self, items):
        # 遍历收集所有用例的信息类
        for i in items:
            node_id = i.nodeid  # 用例唯一id
            first_node = node_id.split('/')[0]  # 一级目录
            # 创建一个类，使用其进行递归，修改其属性 unique_code
            case_info = type('Tmp', (), {
                'case_name': None,
                'first_node': first_node,  # 一级目录
                'second_node': None,  # 二级目录
                'node_id': node_id,  # 节点路径，用例唯一码的原始状态，可用于导入用例
                'case_structure': None,  # 用例的层级结果， 类的链表形式，最外层为模块，最内层为用例函数
                'first_node_doc': '',  # 一级目录描述
                'second_node_doc': '',  # 二级目录描述
            })
            self._case_unique_code(i, case_info)
            self.case_info_list.append(case_info)

    def collect(self):
        # 从 cerebro 中读取判断， 是否收集测试用例到数据库
        if cerebro.is_collect_cases:
            for case_info in self.case_info_list:
                # 对用例node信息tmp.node_path， 进行base64加密， 生成用例唯一编码
                i_unique_code = base64.b64encode(case_info.node_id.encode()).decode()
                case_info.unique_code = i_unique_code
                # diff用例节点
                Q = and_(
                    CaseCollection.unique_code == case_info.unique_code,
                )
                is_exist = self.db_session.query(CaseCollection).filter(Q).all()
                # 不存在用例添加到CaseCollection表
                if not is_exist:
                    new = CaseCollection(**dict(
                        unique_code=case_info.unique_code,
                        case_name=case_info.case_name,
                        first_node_doc=case_info.first_node_doc,
                        second_node_doc=case_info.second_node_doc,
                        first_node=case_info.first_node,
                        second_node=case_info.second_node,

                    ))
                    self.db_session.add(new)
                    self.db_session.commit()
                    self.db_session.close()

    @staticmethod
    def task_report(terminalreporter, engine):
        # 从 cerebro 中读取判断是否上报测试结果
        if cerebro.is_report_test_result:
            # 创建一个mysql链接session
            Session = sessionmaker(engine)
            # 打开会话对象 Session
            db_session = Session()
            # 测试结果数据
            # 异常/成功用例记录
            summary = dict()
            failed = []
            passed = []
            # 成功/失败用例数量
            case_count = terminalreporter._numcollected
            #    成功 / 失败的用例列表
            passed_cases = terminalreporter.stats.get('passed')
            failed_cases = terminalreporter.stats.get('failed')

            pass_count = len(passed_cases) if passed_cases else 0
            fail_count = len(failed_cases) if failed_cases else 0

            # 失败的模块和用例名称
            for i in (passed_cases if passed_cases else []):
                passed.append(base64.b64encode(i.nodeid.encode()).decode())
            # 失败的模块和用例名称
            for i in (failed_cases if failed_cases else []):
                failed.append(base64.b64encode(i.nodeid.encode()).decode())
            summary.update({
                '用例总数': case_count,
                '通过数': pass_count,
                '失败数': fail_count,
                'passed': passed,
                'failed': failed
            })
            new = TaskReport(**dict(
                task_id=uuid.uuid1().hex,
                result=summary,
            ))
            db_session.add(new)
            db_session.commit()
            db_session.close()

==> RealiBoxTestBiz-0.8.6.tar.gz: RealiBoxTestBiz-0.8.6/realiboxtestbiz/utils/test_collection.py <==
'''
收集用例case_collection：
目前用例收集逻辑：
    通过pytest钩子采集每轮执行的用例，通过与 CaseCollection 表中进行diff。
    表中没有则添加。遗留问题：
        需要全量同步方法，否则未调用过的用例不会采集，已删除的用例无记录。目前可以通过刷库解决，
        但是需要在实现前端调用用例之前修复该问题

任务结果上报task_report：

'''
import requests

from ..utils.realiboxtestTables import CaseCollection
from sqlalchemy import and_
from sqlalchemy.orm import sessionmaker
import base64
from _pytest.main import Session
from _pytest.python import Function, Module, Package
from .. import cerebro
from .req_server import save_task_report, case_collect_post, case_collect_list


class Collection:

    def __init__(self, engine):
        # 创建一个mysql链接session
        Session = sessionmaker(engine)
        # 打开会话对象 Session
        self.db_session = Session()
        self.case_info_list = []

    def _case_unique_code(self, obj, tmp):
        '''生成用例唯一码
        params fn：用例函数对象
        params tmp：一个类， 其属性用于记录递归过程用例信息
        '''
        # Session为最后一个节点不处理，跳出递归
        if isinstance(obj, Session):
            return
        # 节点类型
        node_type = None
        # 当obj为Function
        if isinstance(obj, Function):
            node_type = 'function'

        #   当obj是moudule时，为用例文件节点，获取__doc__为用例名称
        if isinstance(obj, Module) and (not tmp.case_name):
            node_type = 'module'
            tmp.case_name = obj.obj.__doc__

        #   当obj是Package时，为路径节点，使用tmp记录其层级关系， 第0级为模块， 获取__doc__为模块名称
        if isinstance(obj, Package):
            node_type = 'package'
            # 判断当前目录是否与一级目录相同
            second_node = '.'.join(obj.nodeid.split('/')[:-2])
            if not second_node == tmp.first_node:
                tmp.second_node = second_node

        # 每进入一个层级则创建一个类，将上一个层级的类加载到这个类的node属性
        current_node = type('Node', (), {
            'next': tmp.case_structure,
            'type': node_type
        })
        # 将当前节点加载到Tmp中
        tmp.case_structure = current_node
        self._case_unique_code(obj.parent, tmp)

    def case_info_list_collect(self, items):
        # 遍历收集所有用例的信息类
        for i in items:
            node_id = i.nodeid  # 用例唯一id
            first_node = node_id.split('/')[0]  # 一级目录
            # 创建一个类，使用其进行递归，修改其属性 unique_code
            case_info = type('Tmp', (), {
                'case_name': '',
                'first_node': first_node,  # 一级目录
                'second_node': '',  # 二级目录
                'node_id': node_id,  # 节点路径，用例唯一码的原始状态，可用于导入用例
                'case_structure': '',  # 用例的层级结果， 类的链表形式，最外层为模块，最内层为用例函数
                'first_node_doc': '',  # 一级目录描述
                'second_node_doc': '',  # 二级目录描述
            })
            self._case_unique_code(i, case_info)
            self.case_info_list.append(case_info)

    def collect(self):
        # 从 cerebro 中读取判断， 是否收集测试用例到数据库
        if cerebro.is_collect_cases:
            for case_info in self.case_info_list:
                # 对用例node信息tmp.node_path， 进行base64加密， 生成用例唯一编码
                i_unique_code = base64.b64encode(case_info.node_id.encode()).decode()
                case_info.unique_code = i_unique_code
                is_exist = case_collect_list(case_info.unique_code).json()
                # 不存在用例添加到CaseCollection表
                if not is_exist:
                    case_collect_post(case_info)

    @staticmethod
    def task_report(terminalreporter):
        # 从 cerebro 中读取判断是否上报测试结果
        if cerebro.is_report_test_result:
            # 测试结果数据
            # 异常/成功用例记录
            summary = dict()
            failed = []
            passed = []
            # 成功/失败用例数量
            case_count = terminalreporter._numcollected
            #    成功 / 失败的用例列表
            passed_cases = terminalreporter.stats.get('passed')
            failed_cases = terminalreporter.stats.get('failed')

            pass_count = len(passed_cases) if passed_cases else 0
            fail_count = len(failed_cases) if failed_cases else 0

            # 失败的模块和用例名称
            for i in (passed_cases if passed_cases else []):
                passed.append(base64.b64encode(i.nodeid.encode()).decode())
            # 失败的模块和用例名称
            for i in (failed_cases if failed_cases else []):
                failed.append(base64.b64encode(i.nodeid.encode()).decode())
            summary.update({
                '用例总数': case_count,
                '通过数': pass_count,
                '失败数': fail_count,
                'passed': passed,
                'failed': failed
            })
            # 保存任务报告
            save_task_report(summary)
